ccc acf_1h_v2.f   Time-stamp: <2013-10-28 14:10:47 hamana>
c 
c   This program computes the halo model prediction for the angular 
c   correlation functions of galaxies for a given selection function "q(z)". 
c   This is for the 1-halo (Poisson) term, use "acf_2h_v2.f" for the 2-halo term.
c
c   USAGE:
c   (1) compile this with a fortran compiler with a compile option that
c       re-defines REAL as DOUBLE PRECISION e.g., -fdefault-real-8, -r8,
c       use an openmp option to run in openmp parallel.
c       example: gfortran -O2 -fdefault-real-8 -fopenmp acf_1h_v2.f -o acf_1h_v2.exe
c                ifort -O2 -r8 -openmp acf_1h_v2.f -o acf_1h_v2.exe
c
c   (2) execute 
c       acf_1h_v2.exe options
c       options are: -h : Hubbple parameter (default = 0.71)
c                   -om : Omega_m (0.27)
c                   -ov : Lambda (0.73) *** FLAT MODEL ONLY ***
c                   -ob : Omega_b (0.045)
c                   -s8 : sigma_8 (0.8)
c                    -f : output file name (data/acf_1h.dat)
c                    -q : selection function (QZ/LBGz4.selfnorm) **
c                   -mn : Mmin in the halo model (1.0e11 [Msun/h])
c                   -m1 : M1 in the halo model (1.0e13[Msun/h] )
c                   -al : alpha in the halo model (0.8)
c
c   *** important note ***
c   The precision of this program in the angular correlation function is about 1e-6.
c   Thus, the ACF is not very accurate on large scales (e.g., theta>0.01deg).
c   You MUST be careful with the outptted ACF on large scales.
c
c (**) format of the selection function data 'q(z)' should be as follows
c     
c z1_min z1_max q(z1)
c z1_min z1_max q(z2)
c ...
c zi_min zi_max q(zi)
c ...
c zn_min zn_max q(zn) 
c    
c where it is assumed that q(z) is given a histogram with n bins.
c For i-bin, z range is between zi_min and zi_max and the q-value
c of that bin is q(zi).
c Be sure  zi_max = z{i+1}_min.
c Also be sure that the q(z) must be normalized to unity,
c that is 
c \sum q(z_i)*(zi_max - zi_min) = 1.
c 
c ---Example---
c ### QZ/LBGz4.selfnorm 
c   3.10000E+00  3.30000E+00  5.05050E-02
c   3.30000E+00  3.50000E+00  7.57576E-02
c   3.50000E+00  3.70000E+00  5.80808E-01
c   3.70000E+00  3.90000E+00  1.18687E+00
c   3.90000E+00  4.10000E+00  1.28788E+00
c   4.10000E+00  4.30000E+00  1.01010E+00
c   4.30000E+00  4.50000E+00  5.80808E-01
c   4.50000E+00  4.70000E+00  2.27273E-01
c 
c
c   HISTORY:
c   2013/10/28 --- Ver 2 by Takashi Hamana (openmp and some modifications)
c   2005/01/25 --- Ver 1.1 by Takashi Hamana (bug-fixing)
c   2005/01/25 --- Ver 1.0 by Takashi Hamana
c
c   Bug report to hamana.tk[at]nao.ac.jp
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      MODULE defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      parameter (irmin=-100)
      parameter (irmax=50)
      parameter (delr=0.25)
      parameter (irall=irmax-irmin+1)
      END MODULE defparams
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      program acf_1h_v2
      use defparams

      implicit real (a-h,o-z)
      implicit integer (i-n)

      common /omegas/ omem,omev
      common /pkshape/ gamfin,atps
      common /defs/ pi,tps,ch0,rn,onethr
      common /st/ p_st,a_st,st_norm
      common /fby/ deltac,deltac2,deltavir3,ga2,zs,rast,ak,as
      common /conc/ b0,c0
      common /an/ an_m1,an_rast,an_alp3,an_alpm13
      common /pairmodel/ paira,pairainv

      external dlda
      external fnu_nmm
      external fnunur3y2

      parameter (ikmin=-100)
      parameter (ikmax=100)
      parameter (delk=0.04)
      parameter (iktot=ikmax-ikmin+1)

      parameter (ikfac=4000)
      parameter (jkmin=ikfac*ikmin)
      parameter (jkmax=ikfac*ikmax)
      parameter (delj=delk/(1.0*ikfac))

      parameter (nphimin=-20)
      parameter (nphimax=0)
      parameter (delphi=0.2)

      parameter (nselfmax=100)

      dimension da(2,nselfmax)
      dimension asf(2,nselfmax)
      dimension delasf(nselfmax)
      dimension qself(2,nselfmax)
      dimension zself(2,nselfmax)
      dimension azself(2,nselfmax)
      dimension pself(nselfmax)


      dimension delp1(ikmin:ikmax)
      dimension delp2(ikmin:ikmax)
      dimension ddelp1(ikmin:ikmax)
      dimension ddelp2(ikmin:ikmax)
      dimension aklog(ikmin:ikmax)
      dimension acp(nphimin:nphimax)

      character*111 filename
      character*111 selfname
      
      character*111 opt,arg

      include 'omp_lib.h'
!$OMP THREADPRIVATE(/fby/)

ccc definitions
      pi=4.0*atan(1.0)
      tps=2.0*pi**2
      e2ten=log(10.0)
      ch0=3000.0
      rn=8.0/ch0
      zero=0.0
      one=1.0
      onethr=1.0/3.0
      rhocr=2.7755 ! *10^11 M_sun h^2 /Mpc^3
      asec2rad=pi/(180.0*60.0*60.0)
      deg2rad=pi/(180.0)
      
      anumax=16.0

ccc flag
      igrfrag=-1

ccc read parameters
      narg=iargc()
c      write(*,*)narg
      hubp=0.71 ! Hubble parameter
      omem=0.27 ! Omega_matter
      omev=0.73 ! Omega_lambda *** FLAT MODEL ONLY ***
      omeb=0.045 ! Omega_b
      sig8=0.8 ! sigma_8
      filename='data/acf_1h.dat' ! out put file name
      selfname='QZ/LBGz4.selfnorm' ! path to the file of the selection function
      ammin=1.0e11 ! Mmin in the halo model [Msun/h]
      am1=1.0e13 ! Mmin in the halo model [Msun/h]
      an_alp=0.8 ! alpha in the halo model
      do i=1,narg
         call getarg(i,opt)
         call getarg(i+1,arg)
         select case (opt)
         case ('-h')
            read(arg,*)hubp
         case ('-om')
            read(arg,*)omem
         case ('-ov')
            read(arg,*)omev
         case ('-ob')
            read(arg,*)omeb
         case ('-s8')
            read(arg,*)sig8
         case ('-f')
            filename=arg
         case ('-q')
            selfname=arg
         case ('-mn')
            read(arg,*)ammin
         case ('-m1')
            read(arg,*)am1
         case ('-al')
            read(arg,*)an_alp
         end select
      enddo

ccc parameters
      omebh2=omeb*hubp**2
      ok=abs(omem+omev-1.0)
      if (ok.ge.1.0e-3) then
         write(*,*)'Flat model only'
         stop
      endif
      rho0=omem*rhocr
      
ccc Gamma factor
      gamfin=exp(omeb+sqrt(2.0*hubp)*omeb/omem)/
     $     (omem*hubp*ch0)

ccc normalization
      call normPk(anrm,sig8)
      atps=anrm/tps

ccc NFW parameters
      c0=8.0
      b0=-0.13

ccc ST parameters
      p_st=0.3
      a_st=0.707

!!! a value
      paira=4.0
      pairainv=1.0/paira
      
!!! M1, Mmin range, alpha
      an_m1=am1/1.0e11
      an_m1=(3.0*an_m1/(4.0*pi*rho0))**onethr

      amassmin=ammin/1.0e11
      an_rast=(3.0*amassmin/(4.0*pi*rho0))**onethr
      rmass=an_rast

      an_alp3=3.0*an_alp
      an_alpm13=3.0*(an_alp-1.0)

ccc read selection function
      nself=0
      checknorm=0.0
      open(10,file=selfname,status='old')
 444  read(10,*,end=445)zsef1,zsef2,psef
      nself=nself+1
      i=nself
      zself(1,i)=zsef1
      zself(2,i)=zsef2
      pself(i)=psef
      checknorm=checknorm+psef*(zsef2-zsef1)

      asf(1,i)=1.0/(1.0+zself(1,i))
      qself(1,i)=pself(i)**2*sqrt(e(asf(1,i)))/asf(1,i)**4
      call qromb(dlda,asf(1,i),one,ss)
      da(1,i)=fk(ss)*ch0

      asf(2,i)=1.0/(1.0+zself(2,i))
      qself(2,i)=pself(i)**2*sqrt(e(asf(2,i)))/asf(2,i)**4
      call qromb(dlda,asf(2,i),one,ss)
      da(2,i)=fk(ss)*ch0

      delasf(i)=(asf(1,i)-asf(2,i))
      goto 444
 445  close(10)
      write(*,*)'### check q(z) ###'
      write(*,*)'# q(z) has',nself,'bins'
      write(*,*)'# int dz q(z) =',checknorm
      checkone=abs(checknorm-1.0)
      if (checkone.lt.1.0e-2) then
         write(*,*)'# normalization OK'
      else
         write(*,*)'# Be sure, we now renormalize it'
         write(*,*)'# q(z) = q(z) /',checknorm
!!!  if q(z) is not normalized, normalizing it
         do i=1,nself
            pself(i)=pself(i)/checknorm
ccc 2005/03/22 bug-fixing begin
            qself(1,i)=qself(1,i)/checknorm**2
            qself(2,i)=qself(2,i)/checknorm**2
ccc 2005/03/22 bug-fixing end
         enddo
      endif
      
      write(*,*)'### OK, start painful integrations ###'
      
ccc power spectrum normalization
      call normPk(anrm,sig8)
      atps=anrm/tps

ccc construct ST mass function
      call int_st(st_norm)

ccc construct sigma-R relation at z=0
      call int_sigmar
   
ccccc angular correlation using limber's equation
c clear arrays
      do k=nphimin,nphimax
         acp(k)=0.0
      enddo

       do i=ikmin,ikmax
          aklog(i)=i*delk
       enddo

      do 31 jz=1,nself ! loop for z
         write(*,*)'#',jz,'/',nself
!!! for 1
         if (jz.eq.1) goto 1001
         if (zself(1,jz).eq.zself(2,jz-1)) then
            goto 1002
         else
            goto 1001
         endif
 1001    as=asf(1,jz)
ccc growth rate
         g=gr(as,igrfrag)
         ga2=(g*as)**2
ccc deltas
         call deltas(as,deltac,deltavir)
         deltac2=deltac**2
         deltavir3=deltavir**onethr
         sigma_mass2=sigma_r(rmass)
         anumin=deltac2/(sigma_mass2*ga2)
         anumin=log(anumin)
         anumid=0.5*(anumin+anumax)
ccc r_ast
         call search_rast(rast,ga2,deltac2)
ccc int dnu f(nu) N(M)/M*const
c         write(*,*)anumin,anumax
         call qromb(fnu_nmm,anumin,anumid,snorm1)
         call qromb(fnu_nmm,anumid,anumax,snorm2)
         snorm=snorm1+snorm2
         snorm=1.0/(1.5*snorm**2)
ccc pk
!$omp parallel private(ss1,ss2)
!$omp+copyin(/fby/)
!$omp do
         do i=ikmin,ikmax
            ak=10.0**(i*delk)
            call qromb(fnunur3y2,anumin,anumid,ss1)
            call qromb(fnunur3y2,anumid,anumax,ss2)            
            delp1(i)=log((ss1+ss2)*ak**2*snorm)
         enddo
!$omp end do
!$omp end parallel
         dmin=(delp1(ikmin+1)-delp1(ikmin))/delk
         dmax=(delp1(ikmax)-delp1(ikmax-1))/delk
         call spline(aklog,delp1,iktot,dmin,dmax,ddelp1)

!!! for 2
 1002    as=asf(2,jz)
ccc growth rate
         g=gr(as,igrfrag)
         ga2=(g*as)**2
ccc deltas
         call deltas(as,deltac,deltavir)
         deltac2=deltac**2
         deltavir3=deltavir**onethr
         sigma_mass2=sigma_r(rmass)
         anumin=deltac2/(sigma_mass2*ga2)
         anumin=log(anumin)
         anumid=0.5*(anumin+anumax)
c         write(*,*)anumin,anumax
ccc r_ast
         call search_rast(rast,ga2,deltac2)
ccc int dnu f(nu) N(M)/M*const
         call qromb(fnu_nmm,anumin,anumid,snorm1)
         call qromb(fnu_nmm,anumid,anumax,snorm2)
         snorm=snorm1+snorm2
         snorm=1.0/(1.5*snorm**2)
ccc pk
!$omp parallel private(ss1,ss2)
!$omp+copyin(/fby/)
!$omp do
         do i=ikmin,ikmax
            ak=10.0**(i*delk)
            call qromb(fnunur3y2,anumin,anumid,ss1)
            call qromb(fnunur3y2,anumid,anumax,ss2)
            delp2(i)=log((ss1+ss2)*ak**2*snorm)
         enddo
!$omp end do
!$omp end parallel
         dmin=(delp2(ikmin+1)-delp2(ikmin))/delk
         dmax=(delp2(ikmax)-delp2(ikmax-1))/delk
         call spline(aklog,delp2,iktot,dmin,dmax,ddelp2)

ccc xi
!$omp parallel private(phi,be,xip1,xip2,akr,wind,xklog,y)
!$omp do
         do kphi=nphimin,nphimax
            phi=10.0**(kphi*delphi)
!!! for 1
            be=tan(deg2rad*phi)*da(1,jz)
            xip1=0.0
            do ik=jkmin,jkmax
               akr=be*10.0**(ik*delj)               
               wind=bessj0(akr)
               xklog=ik*delj
               call splint(aklog,delp1,ddelp1,iktot,xklog,y)
               xip1=xip1+exp(y)*wind
c               write(*,*)phi,wind,xip1
            enddo
!!! for 2
            be=tan(deg2rad*phi)*da(2,jz)
            xip2=0.0
            do ik=jkmin,jkmax
               akr=be*10.0**(ik*delj)
               wind=bessj0(akr)
               xklog=ik*delj
               call splint(aklog,delp2,ddelp2,iktot,xklog,y)
               xip2=xip2+exp(y)*wind
            enddo
!!! sum up
            acp(kphi)=acp(kphi)+0.5*(xip1*qself(1,jz)+
     $           xip2*qself(2,jz))*delasf(jz)
         enddo
!$omp end do
!$omp end parallel
         do i=ikmin,ikmax
            delp1(i)=delp2(i)
         enddo
 31   continue

ccc output
      open(10,file=filename,STATUS= 'UNKNOWN')
      do kphi=nphimin,nphimax
         phi=10.0**(kphi*delphi)
         acp(kphi)=acp(kphi)*delj*e2ten/ch0
         write(10,101)phi,acp(kphi)
      enddo
 101  format(2(1pe13.5))
ccc   write log
      write(10,201)'#################################'
      write(10,201)'# theta[arcsec]; w_1h            '
      write(10,201)'#################################'
      write(10,200)'# Omega_m =',omem
      write(10,200)'# Omega_v =',omev
      write(10,200)'# Omega_b =',omeb
      write(10,200)'#   h_100 =',hubp
      write(10,200)'# sigma_8 =',sig8
      write(10,201)'#################################'
      write(10,200)'# Mmin    =',ammin
      write(10,200)'# M1      =',am1
      write(10,200)'# alpha   =',an_alp
      write(10,201)'#################################'
 200  format(a11,1x,1pe12.5)
 201  format(a33)
      write(10,201)'# q(z): Selection function ######'
      write(10,202)'# file =',selfname
 202  format(a8,1x,a88)
      write(10,201)'# z_1; z_2; q(z)                 '
      do i=1,nself
         write(10,203)'# ',zself(1,i),zself(2,i),pself(i)
      enddo
 203  format(a2,1x,3(1pe13.5))
      close(10)
      end program acf_1h_v2

ccc
      function fnu_nmm(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /fby/ deltac,deltac2,deltavir3,ga2,zs,rast,ak,as
      common /st/ p,a,st_norm
      include 'omp_lib.h'
!$OMP THREADPRIVATE(/fby/)
      anu=exp(x)
      sig=deltac2/anu
      rsigma=sigma2r(sig,ga2)
      fnu_nmm=st_norm*fnu_nu(anu)*anmm(rsigma)
      if (fnu_nm.lt.1.0e-33) fnu_nm=1.0e-33
      return
      end

ccc  N(M)/M=N_*(M/M_*)^alpha/M*const
      function anmm(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /an/ an_m1,an_rast,an_alp3,an_alpm13
      if (r.lt.an_rast) then
         anmm=0.0
      else
         anmm=(r/an_m1)**an_alpm13
      endif
      return
      end
      

ccc  N(M)=N_*(M/M_*)^alpha
      function anm(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /an/ an_m1,an_rast,an_alp3,an_alpm13
      if (r.lt.an_rast) then
         anm=0.0
      else
         anm=(r/an_m1)**an_alp3
      endif
      return
      end

ccc  <N(N-1)>(M)/M
      function annmm(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /pairmodel/ paira,pairainv
      x=anm(r)
      if (x.gt.1.0) then
         annmm=anmm(r)**2
      elseif (x.gt.pairainv) then
         annmm=anmm(r)**2*log(paira*x)/log(paira)
      else
         annmm=0.0
      endif
      return
      end

ccc
      function fnunur3y2(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /fby/ deltac,deltac2,deltavir3,ga2,zs,rast,ak,as
      common /st/ p,a,st_norm
      include 'omp_lib.h'
!$OMP THREADPRIVATE(/fby/)
      anu=exp(x)
      call nu2r_rs(anu,deltac2,deltavir3,
     $     ga2,as,rast,sig,rvir,c_nfw,rs_nfw,rsigma)
      if (anm(rsigma).gt.1.0) then
         fnunur3y2=st_norm*fnu_nu(anu)*rsigma**3*
     $        yfunc(ak,c_nfw,rs_nfw)**2*annmm(rsigma)
         if (fnunur3y2.lt.1.0e-36) fnunur3y2=0.0
      else
         fnunur3y2=st_norm*fnu_nu(anu)*rsigma**3*
     $        abs(yfunc(ak,c_nfw,rs_nfw))*annmm(rsigma)
         if (fnunur3y2.lt.1.0e-33) fnunur3y2=0.0
      endif
ccc
      return
      end
ccc  
      function yfunc(ak,c,rs)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      z=ak*rs
      if (z.gt.1.0e-7) then
         call cisi(z,ciz,siz)
         c1=1.0+c
         zc1=z*c1
         call cisi(zc1,cizc1,sizc1)
         yfunc=(cos(z)*(cizc1-ciz)+sin(z)*(sizc1-siz)
     $        -sin(c*z)/zc1)/(log(c1)-c/c1)
      else 
         yfunc=1.0
      endif
      return
      end
ccc

ccc Sheth & Tormen bias
      function stbias(x,deltac)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /st/ p,a,st_norm
      y=a*x
      stbias=1.0+(y-1.0+2.0*p/(1.0+y**p))/deltac
      return
      end

ccc Sheth & Tormen mass function ! NOT narmalized
      function fnu_nu(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /st/ p,a,st_norm
      y=a*x
      fnu_nu=(1.0+1.0/y**p)*sqrt(y)*exp(-0.5*y)
      return
      end
ccc normalization of Sheth & Tormen mass function    
      subroutine int_st(st_norm)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      external fnu_nu_log
      amin=-80.0
      amax=80.0
      call qromb(fnu_nu_log,amin,amax,ss)
      st_norm=1.0/ss
      return
      end
      function fnu_nu_log(z)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /st/ p,a,st_norm
      x=exp(z)
      y=a*x
      fnu_nu_log=(1.0+1.0/y**p)*sqrt(y)*exp(-0.5*y)
      return
      end

cccccccccc
      subroutine nu2r_rs(anu,deltac2,deltavir3,
     $     ga2,as,rast,sig,rvir,c_nfw,rs_nfw,rsigma)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      sig=deltac2/anu
      rsigma=sigma2r(sig,ga2)
      rvir=rsigma/deltavir3
      c_nfw=amss2c(as,rsigma)
      rs_nfw=rvir/c_nfw
      return
      end
      
      function amss2c(as,r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      common /defs/ pi,tps,ch0,rn,onethr
      common /conc/ b0,c0
ccc Bullock et al model
!!! 2.775e11/1e14=2.775e-3
      amass=2.775e-3*4.0*pi*omem*r**3/3.0
      amss2c=c0*as*amass**b0
      return
      end

ccc delta_c and delta_vir 
ccc VALID only FLAT model
      subroutine deltas(as,deltac,deltavir)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      common /defs/ pi,tps,ch0,rn,onethr
      x=as*(1.0/omem-1.0)**(1.0/3.0)
      deltac=3.0*(12.0*pi)**(2.0/3.0)/20.0*
     $     (1.0-0.00123*log10(1.0+x**3))
      deltavir=18.0*pi**2*(1.0+0.4093*x**2.71572)
      return
      end
      
      subroutine search_rast(rast,ga2,deltac2)
      use defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /sigma_r_relation/ sigmar0,r0,drdsig1,drdsig2
      dimension sigmar0(irall)
      dimension r0(irall)
      sig0=log(deltac2/ga2)
      call hunt(sigmar0,irall,sig0,jlo)
      d=(r0(jlo+1)-r0(jlo))/(sigmar0(jlo+1)-sigmar0(jlo))
      r1=d*(sig0-sigmar0(jlo))+r0(jlo)
      rast=exp(r1)
      return
      end

ccc sigma_R - R relation
      subroutine int_sigmar
      use defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /sigma_r_relation/ sigmar0,r0,drdsig1,drdsig2
      dimension sigmar0(irall)
      dimension r0(irall)
      
      do 10 i=1,irall
         ii=irmax-i+1
         r0(i)=delr*ii
         x=exp(r0(i))
         sigmar0(i)=log(sigma_r(x)) ! Notice sigma_r(x)=sigma_R^2
 10   continue
      drdsig1=(r0(2)-r0(1))/(sigmar0(2)-sigmar0(1))
      drdsig2=(r0(irall)-r0(irall-1))/
     $     (sigmar0(irall)-sigmar0(irall-1))

      return
      end

ccc
      function sigma2r(sig,ga2)
      use defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /sigma_r_relation/ sigmar0,r0,drdsig1,drdsig2
      dimension sigmar0(irall)
      dimension r0(irall)
      sig0=log(sig/ga2)
      if (sig0.le.sigmar0(1)) then
         y=r0(1)+drdsig1*(sig0-sigmar0(1))
      elseif (sig0.ge.sigmar0(irall)) then
         y=r0(irall)+drdsig2*(sig0-sigmar0(irall))
      else
         call hunt(sigmar0,irall,sig0,jlo)
         d=(r0(jlo+1)-r0(jlo))/(sigmar0(jlo+1)-sigmar0(jlo))
         y=d*(sig0-sigmar0(jlo))+r0(jlo)
      endif
      sigma2r=exp(y)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccc
      function sigma_r(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      common /sigmar/rsmooth
      external dsigma_r
      wmin=-30.0
      wmax=40.0
      rsmooth=x/ch0
      call qromb(dsigma_r,wmin,wmax,ss)
      sigma_r=ss
      return
      end   

ccccccccccccccccccccccccccccccccccccccccccccc
      function dsigma_r(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /sigmar/rsmooth
      w=exp(x)
      rk=rsmooth*w
      dsigma_r=delta_lin0(w)*filtTH(rk)**2
      return
      end 

ccccccccccccccccccccccccccccccccccccccccccccc
      function delta_lin0(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /pkshape/ gfin,atps
      q=w*gfin
      delta_lin0=atps*trnf2(q)*w**4
      return
      end 

cccccccccc normalization of power spectrum 
      subroutine normPk(a_norm,sig8)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      external delta
      common /defs/ pi,tps,ch0,rn,onethr
      wmin=-30.0
      wmax=40.0
      call qromb(delta,wmin,wmax,seki)
      y=seki/tps
      a_norm=sig8*sig8/y
      return
      end

ccccccccc
      function delta(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      common /pkshape/ gfin,atps
      w=exp(x)
      q=w*gfin
      r=w*rn
      filt=filtTH(r)
      tranf=trnf2(q)
      delta=w**4*filt*filt*tranf
      return
      end

ccccccccc
      function filtTH(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      filtTH=3.0*(sin(r)-r*cos(r))/r**3
      return
      end

ccccccccc 
      function trnf2(q)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      qq=q*q
      trnf2=0.182628*log(1.0+2.34*q)**2/qq
     $     /sqrt(1.0+3.89*q+259.21*qq
     $     +162.77*qq*q+2027.17*qq*qq)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function gr(a,igrfrag)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      save grnorm
      if (igrfrag.eq.-1) then
         pre=1.0
         et=1.0/e(pre)
         ot=omem*et
         vt=omev*et
         grnorm=0.4*(ot**(0.571428571)-vt+
     $        (1.0+0.5*ot)*(1.0+vt*0.014285714))/ot
         igrfrag=1
      endif
      et=1.0/e(a)
      ot=omem*a*et
      vt=omev*et*a**4
      graw=2.5*ot/(ot**(0.571428571)-vt+
     $     (1.0+0.5*ot)*(1.0+vt*0.014285714))
      gr=graw*grnorm
      return
      end

ccc      
      function e(a)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      e=a*omem+a**4*omev
      return
      end

ccc dl/da
      function dlda(a)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      dlda=1.0/sqrt(e(a))
      return
      end

ccc Comoving D_A
      function fk(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      fk=w
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc integration pakege
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      SUBROUTINE qromb(func,a,b,ss)
      INTEGER JMAX,JMAXP,K,KM
      REAL a,b,func,ss,EPS
      EXTERNAL func
      PARAMETER (EPS=1.e-6, JMAX=40, JMAXP=JMAX+1, K=5, KM=K-1)
CU    USES polint,trapzd
      INTEGER j
      REAL dss,h(JMAXP),s(JMAXP)
      h(1)=1.
      do 11 j=1,JMAX
        call trapzd(func,a,b,s(j),j)
        if (j.ge.K) then
          call polint(h(j-KM),s(j-KM),K,0.,ss,dss)
          if (abs(dss).le.EPS*abs(ss)) return
        endif
        s(j+1)=s(j)
        h(j+1)=0.25*h(j)
11    continue
      write(*,*)'too many steps in qromb'
      stop
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE polint(xa,ya,n,x,y,dy)
      INTEGER n,NMAX
      REAL dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=10)
      INTEGER i,m,ns
      REAL den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.) then
             write(*,*)'failure in polint'
          endif
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE trapzd(func,a,b,s,n)
      INTEGER n
      REAL a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

      SUBROUTINE hunt(xx,n,x,jlo)
      INTEGER jlo,n
      REAL x,xx(n)
      INTEGER inc,jhi,jm
      LOGICAL ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        goto 3
      endif
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          goto 1
        endif
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          goto 2
        endif
      endif
3     if(jhi-jlo.eq.1)return
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      endif
      goto 3
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

ccc 
      SUBROUTINE cisi(x,ci,si)
      INTEGER MAXIT
      REAL ci,si,x,EPS,EULER,PIBY2,FPMIN,TMIN
      PARAMETER (EPS=6.e-8,EULER=.57721566,MAXIT=100,PIBY2=1.5707963,
     *FPMIN=1.e-30,TMIN=2.)
      INTEGER i,k
      REAL a,err,fact,sign,sum,sumc,sums,t,term,absc
      COMPLEX h,b,c,d,del
      LOGICAL odd
      absc(h)=abs(real(h))+abs(aimag(h))
      t=abs(x)
      if(t.eq.0.)then
        si=0.
        ci=-1./FPMIN
        return
      endif
      if(t.gt.TMIN)then
        b=cmplx(1.,t)
        c=1./FPMIN
        d=1./b
        h=d
        do 11 i=2,MAXIT
          a=-(i-1)**2
          b=b+2.
          d=1./(a*d+b)
          c=b+a/c
          del=c*d
          h=h*del
          if(absc(del-1.).lt.EPS)goto 1
11      continue
        write(*,*)'cf failed in cisi'
        stop
1       continue
        h=cmplx(cos(t),-sin(t))*h
        ci=-real(h)
        si=PIBY2+aimag(h)
      else
        if(t.lt.sqrt(FPMIN))then
          sumc=0.
          sums=t
        else
          sum=0.
          sums=0.
          sumc=0.
          sign=1.
          fact=1.
          odd=.true.
          do 12 k=1,MAXIT
            fact=fact*t/k
            term=fact/k
            sum=sum+sign*term
            err=term/abs(sum)
            if(odd)then
              sign=-sign
              sums=sum
              sum=sumc
            else
              sumc=sum
              sum=sums
            endif
            if(err.lt.EPS)goto 2
            odd=.not.odd
12        continue
          write(*,*)'maxits exceeded in cisi'
          stop
        endif
2       si=sums
        ci=sumc+log(t)+EULER
      endif
      if(x.lt.0.)si=-si
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

      FUNCTION bessj0(x)
      REAL bessj0,x
      REAL ax,xx,z
      DOUBLE PRECISION p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,
     *s1,s2,s3,s4,s5,s6,y
      SAVE p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,s1,s2,s3,s4,
     *s5,s6
      DATA p1,p2,p3,p4,p5/1.d0,-.1098628627d-2,.2734510407d-4,
     *-.2073370639d-5,.2093887211d-6/, q1,q2,q3,q4,q5/-.1562499995d-1,
     *.1430488765d-3,-.6911147651d-5,.7621095161d-6,-.934945152d-7/
      DATA r1,r2,r3,r4,r5,r6/57568490574.d0,-13362590354.d0,
     *651619640.7d0,-11214424.18d0,77392.33017d0,-184.9052456d0/,s1,s2,
     *s3,s4,s5,s6/57568490411.d0,1029532985.d0,9494680.718d0,
     *59272.64853d0,267.8532712d0,1.d0/
      if(abs(x).lt.8.)then
        y=x**2
        bessj0=(r1+y*(r2+y*(r3+y*(r4+y*(r5+y*r6)))))/(s1+y*(s2+y*(s3+y*
     *(s4+y*(s5+y*s6)))))
      else
        ax=abs(x)
        z=8./ax
        y=z**2
        xx=ax-.785398164
        bessj0=sqrt(.636619772/ax)*(cos(xx)*(p1+y*(p2+y*(p3+y*(p4+y*
     *p5))))-z*sin(xx)*(q1+y*(q2+y*(q3+y*(q4+y*q5)))))
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

ccc spline fitting
      SUBROUTINE spline(x,y,n,yp1,ypn,y2)
      INTEGER n,NMAX
      REAL yp1,ypn,x(n),y(n),y2(n)
      PARAMETER (NMAX=5000)
      INTEGER i,k
      REAL p,qn,sig,un,u(NMAX)
      if (yp1.gt..99e30) then
        y2(1)=0.
        u(1)=0.
      else
        y2(1)=-0.5
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.
        y2(i)=(sig-1.)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+
     *1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*
     *u(i-1))/p
11    continue
      if (ypn.gt..99e30) then
        qn=0.
        un=0.
      else
        qn=0.5
        un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

      SUBROUTINE splint(xa,ya,y2a,n,x,y)
      INTEGER n
      REAL x,y,xa(n),y2a(n),ya(n)
      INTEGER k,khi,klo
      REAL a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) then
         write(*,*)'bad xa input in splint'
         stop
      endif
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**
     *2)/6.
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
