ccc acf_dm_v2.f   Time-stamp: <2013-10-28 12:03:34 hamana>
c 
c   This program computes (linear and nonlineare) angular correlation 
c   functions of the dark matter for a given selection function "q(z)". 
c   Note that PD96 fitting function for the nonlinear P(k) is adopted.
c
c   USAGE:
c   (1) compile this with a fortran compiler with a compile option that
c       re-defines REAL as DOUBLE PRECISION e.g., -fdefault-real-8, -r8
c       example: gfortran -O2 -fdefault-real-8 acf_dm_v2.f -o acf_dm_v2.exe
c                ifort -O2 -r8 acf_dm_v2.f -o acf_dm_v2.exe
c
c   (2) execute 
c       acf_dm_v2.exe options
c       options are: -h : Hubbple parameter (default = 0.71)
c                   -om : Omega_m (0.27)
c                   -ov : Lambda (0.73) *** FLAT MODEL ONLY ***
c                   -ob : Omega_b (0.045)
c                   -s8 : sigma_8 (0.8)
c                    -f : output file name (data/acf_2h.dat)
c                    -q : selection function (QZ/LBGz4.selfnorm) **
c
c (**) format of the selection function data 'q(z)' should be as follows
c     
c z1_min z1_max q(z1)
c z1_min z1_max q(z2)
c ...
c zi_min zi_max q(zi)
c ...
c zn_min zn_max q(zn) 
c    
c where it is assumed that q(z) is given a histogram with n bins.
c For i-bin, z range is between zi_min and zi_max and the q-value
c of that bin is q(zi).
c Be sure  zi_max = z{i+1}_min.
c Also be sure that the q(z) must be normalized to unity,
c that is 
c \sum q(z_i)*(zi_max - zi_min) = 1.
c 
c ---Example---
c ### QZ/LBGz4.selfnorm 
c   3.10000E+00  3.30000E+00  5.05050E-02
c   3.30000E+00  3.50000E+00  7.57576E-02
c   3.50000E+00  3.70000E+00  5.80808E-01
c   3.70000E+00  3.90000E+00  1.18687E+00
c   3.90000E+00  4.10000E+00  1.28788E+00
c   4.10000E+00  4.30000E+00  1.01010E+00
c   4.30000E+00  4.50000E+00  5.80808E-01
c   4.50000E+00  4.70000E+00  2.27273E-01
c 
c
c   HISTORY:
c   2013/10/26 --- Ver 2 by Takashi Hamana (some modifications)
c   2005/01/25 --- Ver 1.1 by Takashi Hamana (bug-fixing)
c   2005/01/25 --- Ver 1.0 by Takashi Hamana
c
c   Bug report to hamana.tk[at]nao.ac.jp
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      MODULE defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      parameter (ikmin=-500)
      parameter (ikmax=1300)
      parameter (kmax=ikmax-ikmin+1)
      parameter (delk=0.02)
      END MODULE defparams
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      program acf_dm_v2
      use defparams

      implicit real (a-h,o-z)
      implicit integer (i-n)

      common /omegas/ omem,omev
      common /pkshape/ gamfin,atps
      common /defs/ pi,tps,ch0,rn,onethr
      common /xi/ r_xi

      external delta_w_lin
      external delta_w_nl
      external dlda

      parameter (nphimin=-20)
      parameter (nphimax=0)
      parameter (delphi=0.2)
      dimension xili(2,nphimin:nphimax)
      dimension xinl(2,nphimin:nphimax)
      dimension acfli(nphimin:nphimax)
      dimension acfnl(nphimin:nphimax)

      parameter (nselfmax=100)
      dimension da(2,nselfmax)
      dimension asf(2,nselfmax)
      dimension delasf(nselfmax)
      dimension qself(2,nselfmax)
      dimension zself(2,nselfmax)
      dimension azself(2,nselfmax)
      dimension pself(nselfmax)

      character*111 filename
      character*111 selfname
      
      character*111 opt,arg
ccc   integration range
      wmin=-20.0
      wmid=0.0
      wmax=6.0

ccc definitions
      pi=4.0*atan(1.0)
      tps=2.0*pi**2
      ch0=3000.0
      rn=8.0/ch0
      onethr=1.0/3.0
      asec2rad=pi/(180.0*60.0*60.0)
      deg2rad=pi/(180.0)
      one=1.0
      
ccc read parameters
      narg=iargc()
c      write(*,*)narg
      hubp=0.71 ! Hubble parameter
      omem=0.27 ! Omega_matter
      omev=0.73 ! Omega_lambda *** FLAT MODEL ONLY ***
      omeb=0.045 ! Omega_b
      sig8=0.8 ! sigma_8
      filename='data/acf_2h.dat' ! out put file name
      selfname='QZ/LBGz4.selfnorm' ! path to the file of the selection function
      do i=1,narg
         call getarg(i,opt)
         call getarg(i+1,arg)
         select case (opt)
         case ('-h')
            read(arg,*)hubp
         case ('-om')
            read(arg,*)omem
         case ('-ov')
            read(arg,*)omev
         case ('-ob')
            read(arg,*)omeb
         case ('-s8')
            read(arg,*)sig8
         case ('-f')
            filename=arg
         case ('-q')
            selfname=arg
         end select
      enddo
      
ccc parameters
      omebh2=omeb*hubp**2
      ok=abs(omem+omev-1.0)
      if (ok.ge.1.0e-3) then
         write(*,*)'Flat model only'
         stop
      endif

ccc flag
      igrfrag=-1

ccc Gamma factor
      gamfin=exp(omeb+sqrt(2.0*hubp)*omeb/omem)/(omem*hubp*ch0)

ccc read selection function
      nself=0
      checknorm=0.0
      open(10,file=selfname,status='old')
 444  read(10,*,end=445)zsef1,zsef2,psef
      nself=nself+1
      i=nself
      zself(1,i)=zsef1
      zself(2,i)=zsef2
      pself(i)=psef
      checknorm=checknorm+psef*(zsef2-zsef1)

      asf(1,i)=1.0/(1.0+zself(1,i))
      qself(1,i)=pself(i)**2*sqrt(e(asf(1,i)))/asf(1,i)**4
      call qromb(dlda,asf(1,i),one,ss)
      da(1,i)=fk(ss)*ch0

      asf(2,i)=1.0/(1.0+zself(2,i))
      qself(2,i)=pself(i)**2*sqrt(e(asf(2,i)))/asf(2,i)**4
      call qromb(dlda,asf(2,i),one,ss)
      da(2,i)=fk(ss)*ch0

      delasf(i)=(asf(1,i)-asf(2,i))
      goto 444
 445  close(10)
      write(*,*)'### check q(z) ###'
      write(*,*)'# q(z) has',nself,'bins'
      write(*,*)'# int dz q(z) =',checknorm
      checkone=abs(checknorm-1.0)
      if (checkone.lt.1.0e-2) then
         write(*,*)'# normalization OK'
      else
         write(*,*)'# Be sure, we now renormalize it'
         write(*,*)'# q(z) = q(z) /',checknorm
!!!  if q(z) is not normalized, normalizing it
         do i=1,nself
            pself(i)=pself(i)/checknorm
ccc 2005/03/22 bug-fixing begin
            qself(1,i)=qself(1,i)/checknorm**2
            qself(2,i)=qself(2,i)/checknorm**2
ccc 2005/03/22 bug-fixing end
         enddo
      endif
      
      write(*,*)'### OK, start painful integrations ###'

ccc power spectrum normalization
      call normPk(anrm,sig8)
      atps=anrm/tps

ccc PD initialization
      call int_pd

ccccc angular correlation using limber's equation
c clear arrays
      do k=nphimin,nphimax
         acfli(k)=0.0
         acfnl(k)=0.0
      enddo
      
      do 31 jz=1,nself ! loop for z
         write(*,*)'#',jz,'/',nself
!!! for 1
         if (jz.eq.1) goto 1001
         if (zself(1,jz).eq.zself(2,jz-1)) then
            goto 1002
         else
            goto 1001
         endif
 1001    as=asf(1,jz)
ccc growth rate
         g=gr(as,igrfrag)
         ga2=(g*as)**2
         g3=grpd(as)**3
         call pknl(ga2,g3)
ccc xi
         do kphi=nphimin,nphimax
            phi=10.0**(kphi*delphi)
            r_xi=tan(deg2rad*phi)*da(1,jz)
            call qromb(delta_w_lin,wmin,wmid,ss1)
            call qromb(delta_w_lin,wmid,wmax,ss2)
            xili(1,kphi)=(ss1+ss2)*ga2
            call qromb(delta_w_nl,wmin,wmid,ss1)
            call qromb(delta_w_nl,wmid,wmax,ss2)
            xinl(1,kphi)=ss1+ss2
         enddo

!!! for 2
 1002    as=asf(2,jz)
ccc growth rate
         g=gr(as,igrfrag)
         ga2=(g*as)**2
         g3=grpd(as)**3
         call pknl(ga2,g3)
ccc xi
         do kphi=nphimin,nphimax
            phi=10.0**(kphi*delphi)
            r_xi=tan(deg2rad*phi)*da(1,jz)
            call qromb(delta_w_lin,wmin,wmid,ss1)
            call qromb(delta_w_lin,wmid,wmax,ss2)
            xili(2,kphi)=(ss1+ss2)*ga2
            call qromb(delta_w_nl,wmin,wmid,ss1)
            call qromb(delta_w_nl,wmid,wmax,ss2)
            xinl(2,kphi)=ss1+ss2
         enddo

!!! sum up
         do kphi=nphimin,nphimax
            acfli(kphi)=acfli(kphi)+
     $           0.5*(xili(1,kphi)*qself(1,jz)+
     $           xili(2,kphi)*qself(2,jz))*delasf(jz)
            acfnl(kphi)=acfnl(kphi)+
     $           0.5*(xinl(1,kphi)*qself(1,jz)+
     $           xinl(2,kphi)*qself(2,jz))*delasf(jz)
         enddo
 20      continue

         do kphi=nphimin,nphimax
            xili(1,kphi)=xili(2,kphi)
            xinl(1,kphi)=xinl(2,kphi)
         enddo
 31   continue
      
ccc output
      open(10,file=filename,STATUS= 'UNKNOWN')
      do kphi=nphimin,nphimax
         phi=10.0**(kphi*delphi)
         acfli(kphi)=pi*acfli(kphi)
         acfnl(kphi)=pi*acfnl(kphi)
         write(10,101)phi,acfnl(kphi),acfli(kphi)
      enddo
 101  format(3(1pe13.5))

ccc   write log
      write(10,201)'#################################'
      write(10,201)'# theta[degree]; w_dm_nl; w_dm_li'
      write(10,201)'#################################'
      write(10,200)'# Omega_m =',omem
      write(10,200)'# Omega_v =',omev
      write(10,200)'# Omega_b =',omeb
      write(10,200)'#   h_100 =',hubp
      write(10,200)'# sigma_8 =',sig8
      write(10,201)'#################################'
 200  format(a11,1x,1pe12.5)
 201  format(a33)
      write(10,201)'# q(z): Selection function ######'
      write(10,202)'# file =',selfname
 202  format(a8,1x,a88)
      write(10,201)'# z_1; z_2; q(z)                 '
      do i=1,nself
         write(10,203)'# ',zself(1,i),zself(2,i),pself(i)
      enddo
 203  format(a2,1x,3(1pe13.5))
      close(10)

      end program acf_dm_v2

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function delta_w_lin(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      common /xi/ r_xi
      ak=exp(w)
      aw=ak*ch0
      deltak2=delta_lin0(aw)/aw
      akr=r_xi*ak
      wind=bessj0(akr)
      delta_w_lin=deltak2*wind
      return
      end

ccc
      function delta_w_nl(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      common /xi/ r_xi
      ak=exp(w)
      aw=ak*ch0
      deltak2=delta_nl(aw)/aw
      akr=r_xi*ak
      wind=bessj0(akr)
      delta_w_nl=deltak2*wind
      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine int_pd
      use defparams

      implicit real (a-h,o-z)
      implicit integer (i-n)
      real*8 ps1,ps2,ps3,dbletrnf2
      common /pdint/ wl,dell2,a,b,alp,bet,v
      common /pkshape/ gamfin,atps
      
      dimension wl(ikmin:ikmax)
      dimension dell2(ikmin:ikmax)
      dimension a(ikmin:ikmax)
      dimension b(ikmin:ikmax)
      dimension alp(ikmin:ikmax)
      dimension bet(ikmin:ikmax)
      dimension v(ikmin:ikmax)

      delkpm=0.1*delk
      dwp=exp(0.5*delkpm)
      dwm=exp(-0.5*delkpm)

      do ik=ikmin,ikmax
         w=exp(ik*delk)
         wl(ik)=w
         dell2(ik)=delta_lin0(w)
         w2=w*0.5
         w1=w2*dwm
         w3=w2*dwp
         ps1=dbletrnf2(w1*gamfin)*w1
         ps2=dbletrnf2(w2*gamfin)*w2
         ps3=dbletrnf2(w3*gamfin)*w3
         dlplk=(ps3-ps1)/(delkpm*ps2)
         en=1.0/(1.0+dlplk/3.0)
         a(ik)=0.482*en**0.947
         b(ik)=0.226*en**1.778
         alp(ik)=3.310*en**0.244
         bet(ik)=0.862*en**0.287
         v(ik)=11.55*en**0.423
      enddo
      return
      end

ccc
      subroutine pknl(ga2,g3)
      use defparams

      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /pdint/ wl,dell2,a,b,alp,bet,v
      common /nlpk/ deln2,ddeln2,wn,dmin,dmax
      common /defs/ pi,tps,ch0,rn,onethr

      dimension wl(ikmin:ikmax)
      dimension dell2(ikmin:ikmax)
      dimension a(ikmin:ikmax)
      dimension b(ikmin:ikmax)
      dimension alp(ikmin:ikmax)
      dimension bet(ikmin:ikmax)
      dimension v(ikmin:ikmax)
      dimension wn(ikmin:ikmax)
      dimension deln2(ikmin:ikmax)
      dimension ddeln2(ikmin:ikmax)

      do j=ikmin,ikmax
         deln2(j)=fnl(a(j),b(j),alp(j),bet(j),v(j),dell2(j)*ga2,g3)
         wn(j)=wl(j)*(1.0+deln2(j))**onethr
      enddo
      dmin=(deln2(ikmin+1)-deln2(ikmin))/(wn(ikmin+1)-wn(ikmin))
      dmax=(deln2(ikmax)-deln2(ikmax-1))/(wn(ikmax)-wn(ikmax-1))
      call spline(wn,deln2,kmax,dmin,dmax,ddeln2)
      dmin=wn(ikmin+1)*(deln2(ikmin+2)-deln2(ikmin))/
     $     ((wn(ikmin+2)-wn(ikmin))*deln2(ikmin+1))
      dmax=wn(ikmax-1)*(deln2(ikmax)-deln2(ikmax-2))/
     $     ((wn(ikmax)-wn(ikmax-2))*deln2(ikmax-2))
      return
      end

ccc
      function delta_nl(w)
      use defparams
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /nlpk/ deln2,ddeln2,wn,dmin,dmax
      dimension wn(ikmin:ikmax)
      dimension deln2(ikmin:ikmax)
      dimension ddeln2(ikmin:ikmax)
      if (w.ge.wn(ikmax)) then
         y=deln2(ikmax)*(w/wn(ikmax))**dmax
      elseif (w.le.wn(ikmin)) then
         y=deln2(ikmin)*(w/wn(ikmin))**dmin
      else
         call splint(wn,deln2,ddeln2,kmax,w,y)
      endif
      delta_nl=y
      return
      end

ccc
      function fnl(a4,b4,alp4,bet4,v4,x4,g34)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      real a4,b4,alp4,bet4,v4,x4,g34,fnl
      a=a4
      b=b4
      alp=alp4
      bet=bet4
      v=v4
      x=x4
      g3=g34
      fnl8=x*((1.0+b*bet*x+(a*x)**(alp*bet))/
     $     (1.0+((a*x)**alp*g3/(v*sqrt(x)))**bet))**(1.0/bet)
      fnl=real(fnl8)
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccc
      function delta_lin0(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /pkshape/ gfin,atps
      q=w*gfin
      delta_lin0=atps*trnf2(q)*w**4
      return
      end 

cccccccccc normalization of power spectrum 
      subroutine normPk(a_norm,sig8)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      external delta
      common /defs/ pi,tps,ch0,rn,onethr
      coe=0.5/pi**2
      wmin=-10.0
      wmax=10.0
      call qromb(delta,wmin,wmax,seki)
      y=coe*seki
      a_norm=sig8*sig8/y
      return
      end

ccccccccc
      function delta(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /defs/ pi,tps,ch0,rn,onethr
      common /pkshape/ gfin,atps
      w=exp(x)
      q=w*gfin
      r=w*rn
      filt=filtTH(r)
      tranf=trnf2(q)
      delta=w**4*filt*filt*tranf
      return
      end

ccccccccc
      function filtTH(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      filtTH=3.0*(sin(r)-r*cos(r))/r**3
      return
      end

ccccccccc 
      function trnf2(q)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      
      qq=q*q
      trnf2=0.182628*log(1.0+2.34*q)**2/qq
     $     /sqrt(1.0+3.89*q+259.21*qq
     $     +162.77*qq*q+2027.17*qq*qq)

      return
      end

ccccccccc 
      function dbletrnf2(x)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      real x
      
      q=x
      qq=q*q
      dbletrnf2=0.182628*log(1.0+2.34*q)**2/qq
     $     /sqrt(1.0+3.89*q+259.21*qq
     $     +162.77*qq*q+2027.17*qq*qq)

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function gr(a,igrfrag)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      save grnorm
      if (igrfrag.eq.-1) then
         pre=1.0
         et=1.0/e(pre)
         ot=omem*et
         vt=omev*et
         grnorm=0.4*(ot**(0.571428571)-vt+
     $        (1.0+0.5*ot)*(1.0+vt*0.014285714))/ot
         igrfrag=1
      endif
      et=1.0/e(a)
      ot=omem*a*et
      vt=omev*et*a**4
      graw=2.5*ot/(ot**(0.571428571)-vt+
     $     (1.0+0.5*ot)*(1.0+vt*0.014285714))
      gr=graw*grnorm
      return
      end

ccc
      function grpd(a)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ om,ov
      et=1.0/e(a)
      ot=om*a*et
      vt=ov*et*a**4
      grpd=2.5*ot/(ot**(0.571428571)-vt+
     $     (1.0+0.5*ot)*(1.0+vt*0.014285714))
      return
      end  

ccc      
      function e(a)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      e=a*omem+a**4*omev
      return
      end

ccc dl/da
      function dlda(a)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      dlda=1.0/sqrt(e(a))
      return
      end

ccc Comoving D_A
      function fk(w)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      fk=w
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc integration pakege
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      SUBROUTINE qromb(func,a,b,ss)
      INTEGER JMAX,JMAXP,K,KM
      REAL a,b,func,ss,EPS
      EXTERNAL func
      PARAMETER (EPS=1.e-6, JMAX=20, JMAXP=JMAX+1, K=5, KM=K-1)
CU    USES polint,trapzd
      INTEGER j
      REAL dss,h(JMAXP),s(JMAXP)
      h(1)=1.
      do 11 j=1,JMAX
        call trapzd(func,a,b,s(j),j)
        if (j.ge.K) then
          call polint(h(j-KM),s(j-KM),K,0.,ss,dss)
          if (abs(dss).le.EPS*abs(ss)) return
        endif
        s(j+1)=s(j)
        h(j+1)=0.25*h(j)
11    continue
      write(*,*)'too many steps in qromb'
      stop
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE polint(xa,ya,n,x,y,dy)
      INTEGER n,NMAX
      REAL dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=10)
      INTEGER i,m,ns
      REAL den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.) then
             write(*,*)'failure in polint'
             stop
          endif
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE trapzd(func,a,b,s,n)
      INTEGER n
      REAL a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

ccc spline fitting
      SUBROUTINE spline(x,y,n,yp1,ypn,y2)
      INTEGER n,NMAX
      REAL yp1,ypn,x(n),y(n),y2(n)
      PARAMETER (NMAX=5000)
      INTEGER i,k
      REAL p,qn,sig,un,u(NMAX)
      if (yp1.gt..99e30) then
        y2(1)=0.
        u(1)=0.
      else
        y2(1)=-0.5
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.
        y2(i)=(sig-1.)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+
     *1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*
     *u(i-1))/p
11    continue
      if (ypn.gt..99e30) then
        qn=0.
        un=0.
      else
        qn=0.5
        un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE splint(xa,ya,y2a,n,x,y)
      INTEGER n
      REAL x,y,xa(n),y2a(n),ya(n)
      INTEGER k,khi,klo
      REAL a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) then
         write(*,*)'bad xa input in splint'
         stop
      endif
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**
     *2)/6.
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

      FUNCTION bessj0(x)
      REAL bessj0,x
      REAL ax,xx,z
      DOUBLE PRECISION p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,
     *s1,s2,s3,s4,s5,s6,y
      SAVE p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,s1,s2,s3,s4,
     *s5,s6
      DATA p1,p2,p3,p4,p5/1.d0,-.1098628627d-2,.2734510407d-4,
     *-.2073370639d-5,.2093887211d-6/, q1,q2,q3,q4,q5/-.1562499995d-1,
     *.1430488765d-3,-.6911147651d-5,.7621095161d-6,-.934945152d-7/
      DATA r1,r2,r3,r4,r5,r6/57568490574.d0,-13362590354.d0,
     *651619640.7d0,-11214424.18d0,77392.33017d0,-184.9052456d0/,s1,s2,
     *s3,s4,s5,s6/57568490411.d0,1029532985.d0,9494680.718d0,
     *59272.64853d0,267.8532712d0,1.d0/
      if(abs(x).lt.8.)then
        y=x**2
        bessj0=(r1+y*(r2+y*(r3+y*(r4+y*(r5+y*r6)))))/(s1+y*(s2+y*(s3+y*
     *(s4+y*(s5+y*s6)))))
      else
        ax=abs(x)
        z=8./ax
        y=z**2
        xx=ax-.785398164
        bessj0=sqrt(.636619772/ax)*(cos(xx)*(p1+y*(p2+y*(p3+y*(p4+y*
     *p5))))-z*sin(xx)*(q1+y*(q2+y*(q3+y*(q4+y*q5)))))
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

